# include <stdio.h>
# include <bench.h>
# include <proc.io>
# include "field.h"
# include "screen.h"
# include "iodef.h"
# include "sup.h"
# include "passwd.h"

int prev_mode(tab, head, lptr, table, mode)
struct _table *tab;
struct a_line **head, **lptr;
int table, mode;
{
	int stat, try = 0, cnt;
	struct a_line *tmp;

	/*
	 * Cannot read prev if no 
	 * records to start from
	*/
	if (*head == ANULL)
		return (IOGOOD);

	if (!(tab->perms & PERM_INQUIRE))
	{
		errmsg(ERR_NOINQ);
		return (IOPERM);
	}

	/*
	 * Display current record in normal attributes
	 * if list is > 1 deep
	*/
	if (tab->maximum > 1)
		if (tab->dsp_fn != (void (*)())0) 
			(*tab->dsp_fn)(REVVID, mode);

	/*
	 * Make sure we are searching on the right key
	*/
   selectinx(tab->fd, *(tab->keynum), *(tab->keymatch), tab->retry);

	tab->mode = K_UP;

	no_msg = !(tab->messages);

  	stat = prevrec(tab->fd, tab->rec);

	/*
	 * Try & Lock retry times before failing
	 * If No locking is specified, the lock
	 * will just re-establish the current record
	*/
	if (stat != IOTOF)
		while (stat != IOGOOD && ++try <= tab->retry)
		{
			if (stat != IOGOOD && stat != IOLOCKED)
			{
				if (tab->dsp_fn != (void (*)())0) 
					(*tab->dsp_fn)(UNDERLINED, mode);
				return (do_error(stat));
			}

			stat = lock_rec(tab->fd, tab->rec); 
		}
	else
		while (lock_rec(tab->fd, tab->rec) != IOGOOD && ++try <= tab->retry);

	/*
	 * Unlock if previously locked
	*/
	if (tab->retry != 0)
		ulock_rec(tab->fd, tab->rec);

	if (stat != IOGOOD)
	{
		if (tab->dsp_fn != (void (*)())0) 
			(*tab->dsp_fn)(UNDERLINED, mode);
		return (do_error(stat));
	}

	if (*(tab->index) == 0)
	{
		if (tab->maximum > 1)
		{
			/*  ADDED BY TVE (Thurs May 10 1990)....  */
			for (tmp = *lptr, cnt = 0; tmp != ANULL; tmp = tmp->next, cnt++)
				;    /* count # of records below current record */

			if (*head == *lptr && cnt < tab->maximum)  /* at top & room left */
			{
				int saveseq;
				saveseq = *(tab->seq);
				*(tab->seq) = DESCENDING;
				new_record(tab, head, lptr);
				*(tab->seq) = saveseq;
			}
			else
			/*      ........(to here)  TVE  */
				reorder_mode(tab, head, lptr, BACKWARD);

			do_page_mode(tab, head, lptr, mode);
		}
		/*
		 * Display current record hi-lighted
		*/
		else if (tab->dsp_fn != (void (*)())0) 
  			(*tab->dsp_fn)(UNDERLINED, mode);
	}
	else
	{
		*lptr = (*lptr)->prev;
		(*(tab->index))--;

		/*
		 * Display current record hi-lighted
		*/
		if (tab->dsp_fn != (void (*)())0) 
  			(*tab->dsp_fn)(UNDERLINED, mode);
	}

   bytecpy((*lptr)->rec, tab->rec, tab->size);

	return (stat);
}

